package agents;

import java.util.Vector;

import beans.DataPackage;

/**
 * 
 * This class should be extended by all agents.
 * This class implements runnable, so all classes that extends this class must declare a run() method
 * This class contains two Vectors; input and output.
 * input and output are Vectors of Vectors of DataPackage.
 * This means that each class that extend this class can have an undefined number of variables containing an undefined number of elements,
 * in both its input variable and in its output variable
 * 
 * @author Hvard Rykkelid
 *
 */
public abstract class AgentFormat implements Runnable
{

	protected Vector<Vector<DataPackage>> input;
	protected Vector<Vector<DataPackage>> output;
	
	/**
	 * This method is called by the Conveyor class to declare which Vectors this instance of the class should refer to in its input and output
	 * 
	 * @param input - a Vector containing the Vectors of the input-variables
	 * @param output - a Vector containing the Vectors of the output-variables
	 */
	public void init(Vector<Vector<DataPackage>> input, Vector<Vector<DataPackage>> output)
	{
		this.input = input;
		this.output = output;
	}
	
	/**
	 * This method will send a reference to the DataPackage it receives to all the outputs of this agent.
	 * (Reference, not a copy)
	 * 
	 * @param value - the DataPackage that should be sent to all outputs.
	 */
	protected void outputAll(DataPackage value)
	{
		for(int i = 0; i < output.size(); i++)
		{
			output.elementAt(i).addElement(value);
		}
	}
	
}