package agents;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import beans.DataPackage;

import utils.Conveyor;
import utils.TreeNode;

/**
 * 
 * This agent is responsible for filtering away unwanted words and symbols from a text.
 * The words that are unwanted are defined in an xml file and are language-dependent.
 * This class uses the class TreeNode to help storing definitions of unwanted words,
 * and to facilitate efficient checking of words if they are unwanted or not.
 * 
 * This agent takes 2 inputs and one output.
 * 
 * The first input is where FilterAgent receives unfiltered text that should be filtered.
 * The second input is the language-input.  When a new language is set, that information should be sent to the second variable.
 * 
 * The output variable contains the ready filtered text.
 * 
 * @author Hvard Rykkelid
 *
 */
public class FilterAgent extends AgentFormat
{

	private static final String agentSettings = "settings" + File.separator +"agentSettings" + File.separator + "FilterAgent" + File.separator + "settings.txt";

	private static String language = "";

	private static TreeNode root;

	private static String[] general;

	/**
	 * This method will be run when the Thread containing this agentdescription is started
	 */
	public void run()
	{
		// Checks if there is language information ready that should be acted upon
		if(!input.elementAt(1).isEmpty())
		{
			String lang = input.elementAt(1).elementAt(0).getData();
			// Checks if the language allready stored in the language variable is the same as the new input
			// If not, then the variables should be re-initialized for this new language
			if(FilterAgent.language != lang)
			{
				FilterAgent.root = new TreeNode("");
				FilterAgent.language = lang;
				initiateFilter();
			}
		}
		// This is a loop that every iteration checks the global variable Conveyor.finnished if it has been set to true.
		// If it hasn't been set to true, then this agent should still run.
		while(!Conveyor.finished)
		{
			// Checks if there is new information in the inbox.  If there is something it will be processed.
			// If there is no information there, the agent will sleep for a while and make no claim on prosessor-time for a while.
			if(!input.elementAt(0).isEmpty())
			{
				// These two next checks are for sending signals about when the whole program should finish.
				// The first one will only pass on the message and continue working.
				if(input.elementAt(0).elementAt(0).getTag().equals("finish"))
				{
					output.elementAt(0).addElement(input.elementAt(0).remove(0));
					continue;
				}
				// The second one will pass on the message and then terminate this agent.
				if(input.elementAt(0).elementAt(0).getTag().equals("finished"))
				{
					output.elementAt(0).addElement(input.elementAt(0).remove(0));
					break;
				}
				DataPackage local = input.elementAt(0).remove(0);
				String result = local.getData().toLowerCase().replaceAll("\n", " ").replaceAll("\r", " ").replaceAll("\t", " ");
				// After some formating of the text, this for-sentence removes all unwanted symbols.
				for(int i = 0; i < FilterAgent.general.length; i++) result = result.replace(FilterAgent.general[i], "");
				String words[] = result.trim().split(" ");
				result = "";
				// This for-sentence is building the document up again word for word, and it checks each word in the document if it is unwanted or not.
				// It only includes those words that are not on the unwanted list.
				for(int i = 0; i < words.length; i++)
					if(!root.isStopWord(words[i])) result += words[i] + " ";
				// This part of the FilterAgent removes all occurances of double-spaces
				while(result.indexOf("  ") > -1)
				{
					result = result.replaceAll("  ", " ");
				}
				local.setData(result);
				output.elementAt(0).addElement(local);
			} else try
			{
				Thread.sleep(30);
			} catch (InterruptedException e)
			{
			}
			Thread.yield();
		}
	}

	/**
	 * 
	 * This method initiates the filter of FilterAgent based on the contents of the file and the language.
	 * FilterAgent uses TreeNode to build information about what words to exclude, and to efficiently check words afterwards.
	 *
	 */
	private void initiateFilter()
	{
		try
		{
			BufferedReader reader = new BufferedReader(new FileReader(FilterAgent.agentSettings));
			String content = "";
			while(reader.ready()) content += reader.readLine() + "\n";
			reader.close();
			content = content.toLowerCase().replaceAll("\t", "").replaceAll(" ", "");
			FilterAgent.general = content.substring(content.indexOf("<general>") + 10, content.indexOf("</general>")).split("\n");
			String[] stopWords = content.substring(content.indexOf("<" + FilterAgent.language + ">") + 10, content.indexOf("</" + FilterAgent.language + ">")).split("\n");
			for(int i = 0; i < stopWords.length; i++) FilterAgent.root.insert(stopWords[i]);	
		} catch (FileNotFoundException e)
		{	
			e.printStackTrace();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
	}

}