package agents;

import java.util.NoSuchElementException;

import org.htmlparser.parserapplications.StringExtractor;
import org.htmlparser.util.ParserException;

import utils.Conveyor;

import beans.DataPackage;

/**
 * 
 * This agent uses an external library to download the pure text-contents from web-pages.
 * This agent will receive the link to a page, and it will download the contents and
 * it will remove all that is not pure text.
 * 
 * This agent takes one input and one output.
 * 
 * The input is where HTMLFetcherAgent receives the URLs that HTMLFetcherAgent has to fetch and extract the text from.
 * 
 * The output variable contains the downloaded text.
 * 
 * @author Hvard Rykkelid
 *
 */
public class HTMLFetcherAgent extends AgentFormat
{
	
	/**
	 * This method will be run when the Thread containing this agentdescription is started
	 */
	public void run()
	{
		// This is a loop that every iteration checks the global variable Conveyor.finished if it has been set to true.
		// If it hasn't been set to true, then this agent should still run.
		while (!Conveyor.finished)
		{
			// Checks if there is new information in the inbox.  If there is something it will be processed.
			// If there is no information there, the agent will sleep for a while and make no claim on prosessor-time for a while.
			if (!input.elementAt(0).isEmpty())
			{
				DataPackage local = input.elementAt(0).remove(0);
				// These two next checks are for sending signals about when the whole program should finish.
				// The first one will only pass on the message and continue working.
				if(local.getTag().equals("finish"))
				{
					output.elementAt(0).add(local);
					continue;
				}
				// The second one will pass on the message and then terminate this agent.
				if(local.getTag().equals("finished"))
				{
					output.elementAt(0).addElement(local);
					break;
				}
				// This Agent is not able to extract text from pdf documents, so it ignores those documents
				if(local.getData().endsWith("/") || local.getData().toLowerCase().endsWith(".htm") || local.getData().toLowerCase().endsWith(".html")) 
				{
					StringExtractor resource = new StringExtractor(local.getData());
					try
					{
						local.setData(resource.extractStrings(false));
						output.elementAt(0).addElement(local);
					} catch (ParserException e)
					{
					}
					catch (NoSuchElementException e)
					{
					}
					catch (StringIndexOutOfBoundsException e)
					{
					}
					catch (Exception e)
					{
					}
				}
			} else try
			{
				Thread.sleep(30);
			} catch (InterruptedException e)
			{
			}
			Thread.yield();
		}
	}

}