package agents;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JOptionPane;

import utils.Conveyor;
import beans.DataPackage;

/**
 * 
 * PrologAgent is an agent that is meant to start up after all other agents have finished.
 * It takes the knowledge found and makes it into statements usable in the language Prolog.
 * PrologAgent also includes some other Prolog-sentences that makes it possible to reason over the knowledge in Prolog.
 * This agent
 * 
 * PrologAgent takes one input only. That input is knowledge ready to be converted into Prolog-sentences.
 * The ready Prolog-sentences are saved in a file.
 * 
 * @author Hvard Rykkelid
 *
 */
public class PrologAgent extends AgentFormat
{
	
	private static final String writeFile = "output" + File.separator + "PrologAgent" + File.separator + "output.pl";
	private static BufferedWriter writer;
	private static boolean done;
	
	private static String holonymy;
	private static String hyponymy;
	private static String synonymy;
	private static String antonymy;

	/**
	 * This method will be run when the Thread containing this agentdescription is started
	 */
	public void run()
	{
		// Initializing variables
		PrologAgent.done = false;
		PrologAgent.holonymy = "";
		PrologAgent.hyponymy = "";
		PrologAgent.synonymy = "";
		PrologAgent.antonymy = "";
		// This agent is dependent on the variable "Conveyor.finished", but not in the same way as most other agents.
		// This agent will do nothing until after "Conveyor.finished" is set to true.
		// Until "Conveyor.finished" is set to true, this agent will only check the state of "Conveyor.finished" and sleep.
		while(true)
		{
			// Checks the state of "Conveyor.finished".
			if(Conveyor.finished)
			{
				// Checks if there is data ready to be processed, if not the agent will finish up.
				if(input.elementAt(0).isEmpty()) break;
				DataPackage local = input.elementAt(0).remove(0);
				// Checks whether the tag of the data found is equals "finished", if so the agent will finish up.
				if(local.getTag().equals("finished")) break;
				// Checks the tag of the data and stores the data in the corresponding variable as a Prolog-statement.
				if(local.getTag().startsWith("holonymy"))	PrologAgent.holonymy += local.getTag().replaceAll("holonymy", "has_part") + ".\n";
				else if(local.getTag().startsWith("hyponymy")) PrologAgent.hyponymy += local.getTag().replaceAll("hyponymy", "is_a") + ".\n";
				else if(local.getTag().startsWith("synonymy")) PrologAgent.synonymy += local.getTag().replaceAll("synonymy", "equals") + ".\n";
				else if(local.getTag().startsWith("antonymy")) PrologAgent.antonymy += local.getTag().replaceAll("antonymy", "opposite") + ".\n";
			}else try
			{
				Thread.sleep(100);
			} catch (InterruptedException e)
			{
			}
			Thread.yield();
		}
		// Makes sure that this part of the code is only run once
		if(!PrologAgent.done)
		{
			PrologAgent.done = true;
			// Builds the finished Prolog-sentences in the corresponding variables.
			PrologAgent.holonymy += "has_part(Source, Target) :- has_part(Parent, Target), is_a(Source, Parent).\n\n";
			PrologAgent.hyponymy += "\nis_in(Source, Target) :- is_a(Source, Target).\nis_in(Source, Target) :- is_in(Parent, Target), is_a(Source, Parent).\n\n";
			PrologAgent.synonymy += "equals(Source, Target) :- equals(Target, Source).\n\n";
			PrologAgent.antonymy += "opposite(Source, Target) :- opposite(Target, Source).";
			// Tries to write the data to a Prolog-interpretable file.
			try
			{
				PrologAgent.writer = new BufferedWriter(new FileWriter(PrologAgent.writeFile));
				writer.write(PrologAgent.holonymy + PrologAgent.hyponymy + PrologAgent.synonymy + PrologAgent.antonymy);
				PrologAgent.writer.close();
			}
			catch (FileNotFoundException e)
			{
				JOptionPane.showMessageDialog(null, "Output-file for PrologAgent is not accessible. File-location should be " + (new File(PrologAgent.writeFile)).getAbsolutePath());
			}
			catch (IOException e)
			{
				JOptionPane.showMessageDialog(null, "Trouble writing to output-file for PrologAgent. File-location should be " + (new File(PrologAgent.writeFile)).getAbsolutePath());
			}
		}
	}
}