package userinterface;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JLabel;

import beans.DataPackage;

import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import utils.Conveyor;

/**
 * 
 * This class is a runnable userinterface for an automatic knowledge-aquisition system
 * created by Hvard Rykkelid as part of his masters degree in Artificial Intelligence.
 * This system uses multiple agents and swarm-intellect as some of its tools to gather
 * knowledge from the Internet.
 * 
 * @author Hvard Rykkelid
 *
 */
public class UserInterface
{

	// A reference to the settings-file that will be refered to when calling Conveyor.
	// It is possible to have multiple settings-files and send a reference to the one you need.
	private final String agentSettings = "settings" + File.separator + "userSettings.txt";

	private JFrame superFrame = null;  //  @jve:decl-index=0:visual-constraint="86,27"
	private JPanel superContentPane = null;
	private JTextField inputTextField = null;
	private JButton startButton = null;
	private JTextField termCountTextField = null;
	private JLabel inputLabel = null;
	private JLabel termCountLabel = null;
	private UIListener uiListener = new UIListener(this);

	private Conveyor conveyor = new Conveyor(agentSettings);
	private JMenuBar jJMenuBar = null;
	private JMenu fileMenu = null;
	private JMenuItem exitMenuItem = null;

	private SuperFaceAdaptor superFaceAdaptor = new SuperFaceAdaptor(this);

	private JTextField progressTextField = null;

	/**
	 * 
	 * The constructor for UserInterface
	 *
	 */
	public UserInterface()
	{
		getSuperFrame();
	}
	
	/**
	 * This method initializes superFrame	
	 * 	
	 * @return javax.swing.JFrame	
	 */
	private JFrame getSuperFrame()
	{
		if (superFrame == null)
		{
			superFrame = new JFrame();
			superFrame.setSize(new java.awt.Dimension(208,194));
			superFrame.setContentPane(getSuperContentPane());
			superFrame.setTitle("Knowledge");
			superFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			superFrame.setJMenuBar(getJJMenuBar());
			superFrame.addWindowListener(superFaceAdaptor);
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			Dimension frameSize = superFrame.getSize();
			if (frameSize.height > screenSize.height)
			{
				frameSize.height = screenSize.height;
			}
			if (frameSize.width > screenSize.width)
			{
				frameSize.width = screenSize.width;
			}
			superFrame.setLocation((screenSize.width - frameSize.width) / 2,
					(screenSize.height - frameSize.height) / 2);
			superFrame.setVisible(true);
		}
		return superFrame;
	}

	/**
	 * This method initializes superContentPane	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getSuperContentPane()
	{
		if (superContentPane == null)
		{
			termCountLabel = new JLabel();
			termCountLabel.setText("Input number of layers to expand:");
			inputLabel = new JLabel();
			inputLabel.setText("Input startword");
			superContentPane = new JPanel();
			superContentPane.setLayout(new FlowLayout());
			superContentPane.add(inputLabel, null);
			superContentPane.add(getInputTextField(), null);
			superContentPane.add(getStartButton(), null);
			superContentPane.add(termCountLabel, null);
			superContentPane.add(getTermCountTextField(), null);
			superContentPane.add(getProgressTextField(), null);
		}
		return superContentPane;
	}

	/**
	 * This method initializes inputTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getInputTextField()
	{
		if (inputTextField == null)
		{
			inputTextField = new JTextField();
			inputTextField.setPreferredSize(new java.awt.Dimension(100,20));
		}
		return inputTextField;
	}

	/**
	 * This method initializes startButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getStartButton()
	{
		if (startButton == null)
		{
			startButton = new JButton();
			startButton.setText("Start");
			startButton.setActionCommand("start/stop");
			startButton.addActionListener(uiListener);
		}
		return startButton;
	}

	/**
	 * This method initializes termCountTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTermCountTextField()
	{
		if (termCountTextField == null)
		{
			termCountTextField = new JTextField();
			termCountTextField.setPreferredSize(new java.awt.Dimension(40,20));
			termCountTextField.setComponentOrientation(java.awt.ComponentOrientation.RIGHT_TO_LEFT);
			termCountTextField.setText("2");
		}
		return termCountTextField;
	}

	/**
	 * This method initializes jJMenuBar	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */
	private JMenuBar getJJMenuBar()
	{
		if (jJMenuBar == null)
		{
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getFileMenu());
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes fileMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getFileMenu()
	{
		if (fileMenu == null)
		{
			fileMenu = new JMenu();
			fileMenu.setText("File");
			fileMenu.setMnemonic('f');
			fileMenu.add(getExitMenuItem());
		}
		return fileMenu;
	}

	/**
	 * This method initializes exitMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getExitMenuItem()
	{
		if (exitMenuItem == null)
		{
			exitMenuItem = new JMenuItem();
			exitMenuItem.setText("Exit");
			exitMenuItem.setMnemonic('x');
			exitMenuItem.setActionCommand("exit");
			exitMenuItem.addActionListener(uiListener);
		}
		return exitMenuItem;
	}

	/**
	 * This method initializes progressTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getProgressTextField()
	{
		if (progressTextField == null)
		{
			progressTextField = new JTextField();
			progressTextField.setPreferredSize(new java.awt.Dimension(180,20));
			progressTextField.setEnabled(false);
		}
		return progressTextField;
	}

	/**
	 * This is the main method.
	 * The place where the code starts to run.
	 * 
	 * @param args - not used
	 */
	public static void main(String[] args)
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		new UserInterface();
	}
	
	/**
	 * 
	 * A method called if it is decided that the program should stop
	 *
	 */
	private void stop()
	{
		// All elements of this system checks the status of "Conveyor.finished" now and again.
		// That means that it is enough to just set "Conveyor.finished" to "true" for the whole program to stop.
		Conveyor.finished = true;
	}
	
	/**
	 * 
	 * This method is called when the program is ready to start up
	 *
	 */
	private void start()
	{
		// Checks whether the input in the "termCountTextField" is parseable and thus an integer.
		boolean ok = false;
		try
		{
			Integer.parseInt(getTermCountTextField().getText());
			ok = true;
		}
		catch(Exception e)
		{
			JOptionPane.showMessageDialog(null, "Warning:\nYou need to input an integer in number of layers to expand");
		}
		// If it was possible to parse the input in "termCountTextField" the program starts.
		if(ok)
		{
			// The text of the button that said "Start" is changed to say "Stop".
			getStartButton().setText("Stop");
			// Clears all previous data.
			conveyor.resetAllData();
			// Sets data needed for launch
			Conveyor.getVectorWithName("language").addElement(new DataPackage("language", "lang_en"));
			Conveyor.getVectorWithName("output").addElement(new DataPackage("messages", ""));
			Conveyor.getVectorWithName("input").addElement(new DataPackage(getTermCountTextField().getText(), getInputTextField().getText()));
			// Starts "MessageUpdater", an individual thread that reads messages and updates the userinterface.
			Thread temp = new Thread(new MessageUpdater(getProgressTextField(), getStartButton()));
			temp.setDaemon(true);
			temp.setPriority(5);
			temp.start();
			// Runs the "start()"-method in "conveyor", which will initialize and start up all the agents and all the variables.
			conveyor.start();
		}
	}
	
	/**
	 * 
	 * This is the method that will be executed every time the "startButton" is pressed.
	 *
	 */
	public void startStop()
	{
		// Figures out if the current command is to start or stop the program and runs the appropriate method.
		if(getStartButton().getText().equals("Stop")) stop();
		else start();
	}

	/**
	 * 
	 * This method is run if the window is asked to close.
	 *
	 */
	public void systemExit()
	{
		// Checks if the user really want to exit.
		int result = JOptionPane.showConfirmDialog(superFrame,
				"Are you sure you want to exit?", "Window closing",
				JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
		// If yes, then the system exits.
		if (result == JOptionPane.YES_OPTION)
		{
			System.exit(0);
		}
	}			
	
}

/**
 * 
 * This internal class listens for "ActionEvent"s and runs apropriate code.
 * 
 * @author Hvard Rykkelid
 *
 */
class UIListener implements ActionListener
{
	
	UserInterface adaptee;
	
	/**
	 * 
	 * The constructor
	 * 
	 * @param adaptee - a reference to the userinterface
	 */
	UIListener(UserInterface adaptee)
	{
		this.adaptee = adaptee;
	}

	/**
	 * 
	 * This method checks the message in the "ActionEvent"s received and acts accordingly.
	 * 
	 * @param e - the ActionEvent
	 */
	public void actionPerformed(ActionEvent e)
	{
		if(e.getActionCommand().equals("start/stop")) adaptee.startStop();
		if(e.getActionCommand().equals("exit")) adaptee.systemExit();
	}
}

/**
 * 
 * This internal class listens for events trying to close the userinterface window.
 * When that happens this class will call the method "systemExit()" in "UserInterface".
 * 
 * @author Hvard Rykkelid
 *
 */
class SuperFaceAdaptor extends WindowAdapter
{

	UserInterface	adaptee;

	/**
	 * 
	 * The constructor
	 * 
	 * @param adaptee - a reference to the userinterface
	 */
	SuperFaceAdaptor(UserInterface adaptee)
	{
		super();
		this.adaptee = adaptee;
	}

	/**
	 * 
	 * This method calls "UserInterface.systemExit()" when an event tries to close the userinterface-window.
	 * 
	 * @param e - the ActionEvent
	 */
	public void windowClosing(WindowEvent e)
	{
		adaptee.systemExit();
	}

}

/**
 * 
 * This internal class is meant to run as an independent thread to update messages to the user.
 * While the program is running, this thread will often check for new messages to post.
 * 
 * @author Hvard Rykkelid
 *
 */
class MessageUpdater implements Runnable
{
	
	JTextField textField;
	JButton button;
	Vector<DataPackage> messages;
	
	/**
	 * 
	 * The constructor
	 * 
	 * @param textField - a reference to the text-field where messages should be posted
	 * @param button - a reference to the button that should change text between "start" and "stop"
	 */
	MessageUpdater(JTextField textField, JButton button)
	{
		this.textField = textField;
		this.button = button;
		messages = Conveyor.getVectorWithName("output");
	}

	/**
	 * This method will be run when the Thread containing this class-description is started
	 */
	public void run()
	{
		// This is a loop that every iteration checks the global variable Conveyor.finished if it has been set to true.
		// If it hasn't been set to true, then this thread should still run.
		while(!Conveyor.finished)
		{
			// Setting the message-field to show the information found in the message-inbox.
			textField.setText(messages.elementAt(0).getData());
			Thread.yield();
			try
			{
				Thread.sleep(100);
			} catch (InterruptedException e)
			{
			}
		}
		// When "Conveyor.finished" has been set to true, the text on the button should be set to "Start".
		button.setText("Start");
	}
	
}